# pcorrcoef: Partial correlation matrix  

import numpy as np

def pcorrcoef(X):

    # Center X
    X = X - np.mean(X,axis=0)
    
    #%% Correalation matrix
    
    dataset_corr = np.corrcoef(X.T)

    
    #%% Partial correlation matrix
    
    #inverse of the correlation matrix
    corr_inv = np.linalg.inv(dataset_corr)
    nrow_inv_corr, ncol_inv_corr = dataset_corr.shape
    
    #partial correlation matrix
    A = np.ones((nrow_inv_corr,ncol_inv_corr))
    for i in range(0,nrow_inv_corr,1):
        for j in range(i,ncol_inv_corr,1):
            if i !=j:
                #above the diagonal
                A[i,j] = - (corr_inv[i,j]) / (np.sqrt(corr_inv[i,i] * corr_inv[j,j]))
                #below the diagonal
                A[j,i] = A[i,j]
            
            
    return A


# correlated_matrix : Create (normal) random matrix with correlated columns

import numpy as np
import matplotlib.pyplot as plt

def correlated_matrix(obs,C):

    # C is a variance-covariance matrix
    # Examples:
    # C = np.array([[3, 0.9],[0.9, 1]])
    # C = 0.5*np.ones((40,40)) + np.eye(40)
    
    X = np.random.randn(obs,C.shape[0])
    
    #C = np.array([[3, 0.9],[0.9, 1]])
    
    Y = X@C

    return Y


import scipy
import scipy.cluster.hierarchy as sch
import pandas as pd
import numpy as np

def corr_reorder(corr_array, inplace=False):
    """
    Rearranges the correlation matrix, corr_array, so that groups of highly 
    correlated variables are next to eachother 
    
    Parameters
    ----------
    corr_array : pandas.DataFrame or numpy.ndarray
        a NxN correlation matrix 
        
    Returns
    -------
    pandas.DataFrame or numpy.ndarray
        a NxN correlation matrix with the columns and rows rearranged
    """
    pairwise_distances = sch.distance.pdist(corr_array)
    linkage = sch.linkage(pairwise_distances, method='complete')
    cluster_distance_threshold = pairwise_distances.max()/2
    idx_to_cluster_array = sch.fcluster(linkage, cluster_distance_threshold, 
                                        criterion='distance')
    idx = np.argsort(idx_to_cluster_array)
    
    if not inplace:
        corr_array = corr_array.copy()
    
    if isinstance(corr_array, pd.DataFrame):
        return corr_array.iloc[idx, :].T.iloc[idx, :]
    return corr_array[idx, :][:, idx], idx






# Autoscale: center and scale to unit variance

import numpy as np

def autoscale(X, *args):
    '''
    
    Autoscale: center and scale to unit variance
    [X_auto, mX, sX] = autoscale(X)
    
    INPUT
    X [n x k] <numpy.ndarray>
        spectra
        n samples
        k variables
    *args
        optional inputs
        mX [1 x k] 
            mean of all variables - computed beforehand
        sX [1 x k] 
            standard deviation of all variables - computed beforehand       
    
    OUTPUT
    X_auto [n x k] 
        preprocessed spectra
    mX [1 x k] 
        mean of all variables
    sX [1 x k] 
        standard deviation of all variables        
    
    '''
    
    
    varargin = args
    n_params = len(varargin)

    if n_params == 2:
        mx = varargin[0]
        stdx = varargin[1]
        mx = np.squeeze(mx)
        stdx = np.squeeze(stdx)
        
    else:
        mx = np.nanmean(X, axis=0)
        stdx = np.nanstd(X, axis=0)     
    
    
    m,n = X.shape
    
    # Deal with NaN values occuring when std = 0
    for i in range(X.shape[1]):
        if stdx[i] == 0:
            X[:,i] = np.zeros((X.shape[0]))
            
        else:
            X[:,i] = (X[:,i] - mx[i])/stdx[i]
    
    mx = mx[np.newaxis]
    stdx = stdx[np.newaxis]
    
    return X, mx, stdx


# reset: resent Python console



def reset():
    '''
    
    Reset: resent Python console
    - clear console
    - delete variables
    - close windows
    reset(X)
    
    INPUT
    none
    
    OUTPUT
    none 
    
    '''
    
    
    # Clear console
    
    # A good start, but causes problem when printing subsequent results
    #import os
    #os.system('cls' if os.name=='nt' else 'clear')
    
    # Workaround
    print('\n'*100)
    
    
    # Delete variables
    from IPython import get_ipython
    get_ipython().magic('reset -sf')
    
    
    # Close windows
    import matplotlib.pyplot as plt
    plt.close('all')
    
    return



# axaline: Arbitrary line with intercept and slope

import matplotlib.pyplot as plt 
import numpy as np    

def axaline(slope, intercept,style=':r',label='name'):
   
    """
    axaline: plot a line from slope and intercept
    # Acts like axhline or axvline
    # Works for 2D plots
    
    axaline(slope, intercept,style=':r',label='name')
    
    INPUT
    slope [1 x 1] <numpy.ndarray> 
    intercept [1 x 1] <numpy.ndarray>   
    stype <text>
        optional input
    label <text>
        optional input    
    
    """
    
    
    
    axes = plt.gca()
    x_vals = np.array(axes.get_xlim())
    y_vals = intercept + slope * x_vals
    plt.plot(x_vals, y_vals, style, label=label)



from RG.xlsread import *
from RG.regress import *
from RG.reset import *

from RG.PCA import *
from RG.PLS import *
from RG.VIP import *
from RG.mbPLS import *
from RG.cPCA import *
from RG.LDA import *
from RG.PCA_ellipse import *
from RG.ICA import *
from RG.MCR import *

from RG.colorspectra import *
from RG.axaline import *
from RG.lags import *
from RG.DTW import *
from RG.DTW_batch import *

from RG.correlated_matrix import *
from RG.corr_reorder import *
from RG.pcorrcoef import *
from RG.R2 import *
from RG.normplot import *
from RG.VIF import *

from RG.center import *
from RG.autoscale import *
from RG.baseline import *
from RG.SNV import *
from RG.MSC import *
from RG.EMSC import *
from RG.savitzky_golay import *
from RG.detrend import *

from RG.email import *













# MCR: Multivariate Curve Resolution (by ALS: alternating least squares)

import numpy as np

def MCR(X,nbPC):

    # Normalize spectra (each spectrum has a sum=1)
    for i in range(X.shape[0]):
        X[i,:] = X[i,:]/np.sum(X[i,:])
    
    x0, x1 = X.shape
    
    # Initialize C
    np.random.seed()
    C = np.random.rand(x0, nbPC)/nbPC
    C_temp = np.random.rand(x0, nbPC)/nbPC
    error = 1e10
    
    while error > 1E-7:
    
        # Compute S
        S = np.linalg.inv(C.T@C)@C.T@X

        # Correct negative spectra
        S[S < 0] = 0
        
        # Normalize spectra (each spectrum has a sum=1)
        for i in range(nbPC):
            S[i,:] = S[i,:]/np.sum(S[i,:])
        
        # Compute C
        C = X@S.T@np.linalg.inv(S@S.T)
        
        # Correct impossible compositions
        C[C < 0] = 0 
        C[C > 1] = 1                  
                    
        # Normalize composition (each timepoint has a sum=1)
        for i in range(C.shape[0]):
            C[i,:] = C[i,:]/np.sum(C[i,:])
        
        # Check c convergence --------------------
        error = np.sum(np.power((C-C_temp),2)) # Squared error
        C_temp = C
        # ----------------------------------------     
    
    return C,S


# DTW_batch: Dynamic time warping (DTW) for a 3D datacube - batch process

import numpy as np
import RG as rg
from skimage.transform import resize

def DTW_batch(X):
    '''
    
    DTW_batch: Dynamic time warping (DTW) for a 3D datacube - batch process
    [X_align] = DTW_batch(X)
    
    Important: the first variable is used to align all the others
    
    INPUT
    X [a x b x c] <numpy.ndarray>
        3D datacube, typical of a batch process
        a times
        b batches
        c variables
    
    OUTPUT
    X_align [a x b x c] 
        aligned data
       
    '''
    
    # Alignment is based on the first variable
    for i in range(1,X.shape[1]): # for each batch (column of X, excluding x0, the reference)
        x0 = X[:,0,0]
        xi = X[:,i,0]
        distance, path = rg.DTW(x0, xi)
        path = np.array(path) # Path suggested by DWT to align xi and x0
        
        # Align x0 (ref batch)
        x0 = x0[path[:,0]]
        x0 = x0[np.newaxis]
        x0 = resize(x0, (1,X.shape[0]))
        X[:,0,0] = np.squeeze(x0)
    
        # Align other variables linked to x0
        for k in range(1,X.shape[2]):
            xk = X[:,0,k]
            xk = xk[path[:,0]]
            xk = xk[np.newaxis]
            xk = resize(xk, (1,X.shape[0]))
            X[:,0,k] = np.squeeze(xk)
    
        # Align xi
        xi = xi[path[:,1]]
        xi = xi[np.newaxis]
        xi = resize(xi, (1,X.shape[0]))
        X[:,i,0] = np.squeeze(xi)     
        
        # Align other variables linked to xi
        for k in range(1,X.shape[2]):
            xk = X[:,i,k]
            xk = xk[path[:,1]]
            xk = xk[np.newaxis]
            xk = resize(xk, (1,X.shape[0]))
            X[:,i,k] = np.squeeze(xk)
        
        # Align all other batches between 0 and i
        # These have already been aligned with x0, just apply the alignment for x0
        if i>1:
            for j in range(1,i):
                xj = X[:,j,0]
                xj = xj[path[:,0]]
                xj = xj[np.newaxis]
                xj = resize(xj, (1,X.shape[0]))
                X[:,j,0] = np.squeeze(xj)
                
                # Align other variables linked to x0
                for k in range(1,X.shape[2]):
                    xk = X[:,j,k]
                    xk = xk[path[:,0]]
                    xk = xk[np.newaxis]
                    xk = resize(xk, (1,X.shape[0]))
                    X[:,j,k] = np.squeeze(xk)  

    return X


# email: send email via Python

from email.mime.text import MIMEText
from email.header import Header
import smtplib

def email(user,pwd,to,title,text):
    """
    email: send email via Python
    
    INPUT
    user (string) : enter sender email
    pwd (string): enter sender email password
    to (string) : enter reciever email
    title (string) : title of the email
    text (string) : body of the email
    
    OUTPUT
    None
    
    EXAMPLE
    
    user='ryan.gosselin@usherbrooke.ca'
    pwd=input('Input Outlook password: ')
    to = 'ryan.gosselin@gmail.com'
    title = 'Test title'
    
    name = 'John'
    g1 = 5
    g2 = 6
    
    text = '\nBonjour {},'\
    '\n\nVoici un survol de tes résultats à l\'examen 1.'\
    '\n\nQuestion 1: {:.2f} /5.'\
    '\nQuestion 2: {:.2f} /5.'\
    '\n\nRyan'.format(name,g1,g2)
    
    """
    
    server = smtplib.SMTP('smtp.office365.com', 587) #to work with outlook
    server.ehlo()
    server.starttls()
    server.login(user, pwd)
    
    body = MIMEText(text, 'plain', 'utf-8')
    body['From'] = user
    body['To'] = to
    body['Subject'] = Header(title, 'utf-8')
    try:
        server.sendmail(user, [to], body.as_string())
        print('\nemail was sent')
    except:
        print('\nerror')
    server.quit()